@@ -12,10 +12,11 @@
 # - test_Poly_interface(R)
 # - test_MPoly_interface(R)
 # - test_MatSpace_interface(R)
-# - test_MatAlgebra_interface(R)
+# - test_MatRing_interface(R)
 
+@nospecialize
 
-function test_NCRing_interface(R::AbstractAlgebra.NCRing; reps = 50)
+function test_NCRing_interface(R::AbstractAlgebra.NCRing; reps = 15)
 
    T = elem_type(R)
 
@@ -219,7 +220,7 @@ function test_NCRing_interface(R::AbstractAlgebra.NCRing; reps = 50)
 end
 
 
-function test_Ring_interface(R::AbstractAlgebra.Ring; reps = 50)
+function test_Ring_interface(R::AbstractAlgebra.Ring; reps = 15)
 
    T = elem_type(R)
 
@@ -284,7 +285,7 @@ function test_Ring_interface(R::AbstractAlgebra.Ring; reps = 50)
    return nothing
 end
 
-function test_Field_interface(R::AbstractAlgebra.Field; reps = 50)
+function test_Field_interface(R::AbstractAlgebra.Field; reps = 15)
 
    T = elem_type(R)
 
@@ -312,7 +313,7 @@ function test_Field_interface(R::AbstractAlgebra.Field; reps = 50)
    return nothing
 end
 
-function test_EuclideanRing_interface(R::AbstractAlgebra.Ring; reps = 20)
+function test_EuclideanRing_interface(R::AbstractAlgebra.Ring; reps = 15)
 
    T = elem_type(R)
 
@@ -404,7 +405,7 @@ function test_EuclideanRing_interface(R::AbstractAlgebra.Ring; reps = 20)
 end
 
 
-function test_Poly_interface(Rx::AbstractAlgebra.PolyRing; reps = 30)
+function test_Poly_interface(Rx::AbstractAlgebra.PolyRing; reps = 10)
 
    T = elem_type(Rx)
 
@@ -511,7 +512,7 @@ function test_Poly_interface(Rx::AbstractAlgebra.PolyRing; reps = 30)
 end
 
 
-function test_MPoly_interface(Rxy::AbstractAlgebra.MPolyRing; reps = 30)
+function test_MPoly_interface(Rxy::AbstractAlgebra.MPolyRing; reps = 10)
 
    # for simplicity, these tests for now assume exactly two generators
    @assert ngens(Rxy) == 2
@@ -650,7 +651,7 @@ function test_MPoly_interface(Rxy::AbstractAlgebra.MPolyRing; reps = 30)
 end
 
 
-function test_MatSpace_interface(S::MatSpace; reps = 20)
+function test_MatSpace_interface(S::MatSpace; reps = 10)
 
    ST = elem_type(S)
    R = base_ring(S)
@@ -761,12 +762,51 @@ function test_MatSpace_interface(S::MatSpace; reps = 20)
           # TODO: ! variants (such as `swap_cols!` etc.) of all of the above
       end
 
+      if nrows(S) == ncols(S)
+         @testset "Determinant" begin
+            @test isone(det(one(S))) # should always hold, even for 0x0
+            @test nrows(S) == 0 || iszero(det(zero(S)))
+
+            @test isone(AbstractAlgebra.det_df(one(S))) # should always hold, even for 0x0
+            @test nrows(S) == 0 || iszero(AbstractAlgebra.det_df(zero(S)))
+
+            if base_ring_type(S) <: Field
+               @test isone(AbstractAlgebra.det_fflu(one(S))) # should always hold, even for 0x0
+               @test nrows(S) == 0 || iszero(AbstractAlgebra.det_fflu(zero(S)))
+            end
+            if base_ring_type(S) <: PolyRingElem && is_domain_type(base_ring_type(S))
+               @test isone(AbstractAlgebra.det_interpolation(one(S))) # should always hold, even for 0x0
+               @test nrows(S) == 0 || iszero(AbstractAlgebra.det_interpolation(zero(S)))
+            end
+
+            for k in 1:reps
+               a = generate_element(S)::ST
+               b = generate_element(S)::ST
+               A = deepcopy(a)
+               B = deepcopy(b)
+
+               da = det(a)
+               db = det(b)
+               @test det(a*b) == da*db
+               @test da == AbstractAlgebra.det_df(a)
+               if base_ring_type(S) <: Field
+                  @test da == AbstractAlgebra.det_fflu(a)
+               end
+               if base_ring_type(S) <: PolyRingElem && is_domain_type(base_ring_type(S))
+                  @test da == AbstractAlgebra.det_interpolation(a)
+               end
+
+               @test a == A
+               @test b == B
+             end
+         end
+      end
    end
 
    return nothing
 end
 
-function test_MatAlgebra_interface(S::MatRing; reps = 20)
+function test_MatRing_interface(S::MatRing; reps = 15)
 
    ST = elem_type(S)
    R = base_ring(S)
@@ -803,6 +843,9 @@ function test_MatAlgebra_interface(S::MatRing; reps = 20)
       end
 
       @testset "Determinant" begin
+         @test isone(det(one(S))) # should always hold, even for 0x0
+         @test degree(S) == 0 || iszero(det(zero(S)))
+
          for k in 1:reps
             a = generate_element(S)::ST
             b = generate_element(S)::ST
@@ -818,7 +861,7 @@ function test_MatAlgebra_interface(S::MatRing; reps = 20)
    return nothing
 end
 
-function test_Ring_interface_recursive(R::AbstractAlgebra.Ring; reps = 50)
+function test_Ring_interface_recursive(R::AbstractAlgebra.Ring; reps = 15)
    test_Ring_interface(R; reps = reps)
    Rx, _ = polynomial_ring(R, :x)
    test_Poly_interface(Rx, reps = 2 + fld(reps, 2))
@@ -826,7 +869,7 @@ function test_Ring_interface_recursive(R::AbstractAlgebra.Ring; reps = 50)
    test_MPoly_interface(Rxy, reps = 2 + fld(reps, 2))
    for d in 0:3
      S = matrix_ring(R, d)
-     test_MatAlgebra_interface(S, reps = 2 + fld(reps, 8))
+     test_MatRing_interface(S, reps = 2 + fld(reps, 8))
    end
    for (r,c) in ( (0,0), (1,0), (0,2), (2,2), (2,3), (3,2) )
      S = matrix_space(R, r,c)
@@ -834,7 +877,9 @@ function test_Ring_interface_recursive(R::AbstractAlgebra.Ring; reps = 50)
    end
 end
 
-function test_Field_interface_recursive(R::AbstractAlgebra.Field; reps = 50)
+function test_Field_interface_recursive(R::AbstractAlgebra.Field; reps = 15)
    test_Ring_interface_recursive(R, reps = reps)
    test_Field_interface(R, reps = reps)
 end
+
+@specialize
