@@ -1,7 +1,9 @@
 @doc raw"""
- AbstractAlgebra is a pure Julia package for computational abstract algebra.
+AbstractAlgebra is a pure Julia package for computational abstract algebra.
 
- For more information see https://github.com/Nemocas/AbstractAlgebra.jl
+Repository: <https://github.com/Nemocas/AbstractAlgebra.jl>
+
+Documentation: <https://nemocas.github.io/AbstractAlgebra.jl>
 """
 module AbstractAlgebra
 
@@ -196,6 +198,16 @@ include("ConcreteTypes.jl")
 #
 ###############################################################################
 
+# prevent the default Julia methods for iszero and isone from being invoked on
+# one of our parent objects: the default implementations are
+#   iszero(x) = x == zero(x)    and    isone(x) = x == one(x)
+# which for a parent object will always return false, which usually is not
+# what the user hoped for. So let's rather throw an error and leave it to
+# subtypes to implement better methods (e.g. `iszero(::Ring)` or potentially
+# isone(::Group) come to mind).
+iszero(S::Set) = throw(NotImplementedError(:iszero, S))
+isone(S::Set) = throw(NotImplementedError(:isone, S))
+
 include("fundamental_interface.jl")
 include("misc/VarNames.jl")
 include("Infinity.jl")
@@ -373,6 +385,34 @@ end
 include("misc/ProductIterator.jl")
 include("misc/Evaluate.jl")
 
+@doc raw"""
+    is_pairwise(f, a::AbstractVector)
+
+Return whether `f(a[i], a[j])` is true for all `i != j`. It is assumed that `f`
+is symmetric.
+
+# Examples
+
+```jldoctest
+julia> is_pairwise(!is_associated, [2, -3, 5])
+true
+
+julia> is_pairwise(!is_associated, [2, -3, -2])
+false
+```
+"""
+function is_pairwise(fun, arr::AbstractVector)
+  for i in 1:length(arr)
+    for j in 1:(i - 1)
+      if !fun(arr[i], arr[j])
+        return false
+      end
+    end
+  end
+  return true
+end
+
+
 ###############################################################################
 #
 #   methods to inquire about known properties of objects
