@@ -45,6 +45,7 @@ mpoly_type(::Type{T}) where T<:RingElement = Generic.MPoly{T}
 mpoly_type(::Type{S}) where S<:Ring = mpoly_type(elem_type(S))
 mpoly_type(x) = mpoly_type(typeof(x)) # to stop this method from eternally recursing on itself, we better add ...
 mpoly_type(::Type{T}) where T = throw(ArgumentError("Type `$T` must be subtype of `RingElement`."))
+mpoly_type(T::Type{Union{}}) = throw(MethodError(mpoly_type, (T,)))
 
 @doc raw"""
     mpoly_ring_type(::Type{T}) where T<:RingElement
@@ -497,11 +498,11 @@ function is_monomial(x::MPolyRingElem{T}) where T <: RingElement
 end
 
 function exponent_vector!(e::Vector{S}, a::MPolyRingElem{T}, i::Int) where {T <: RingElement, S}
-   return S.(exponent_vector(a, i))
+   return [S(x) for x in exponent_vector(a, i)]
 end
 
 function exponent_vector(::Type{Vector{S}}, a::MPolyRingElem{T}, i::Int) where {T <: RingElement, S}
-   return S.(exponent_vector(a, i))
+   return [S(x) for x in exponent_vector(a, i)]
 end
 
 function coeff!(c::T, a::MPolyRingElem{T}, i::Int) where {T <: RingElement}
@@ -915,7 +916,7 @@ end
 
 function evaluate(a::MPolyRingElem{T}, vals::Vector) where {T <: RingElement}
    @req length(vals) == nvars(parent(a)) "Incorrect number of values in evaluation"
-   newvals = parent(a).(vals)
+   newvals = map(parent(a), vals)
    if typeof(vals) == typeof(newvals)
      throw(NotImplementedError(:evaluate, a, vals))
    end
@@ -1130,6 +1131,19 @@ function evaluate(a::MPolyRingElem{T}, vals::Vector{U}) where {T <: RingElement,
    return a(vals...)
 end
 
+function (a::MPolyRingElem)(;kwargs...)
+   ss = symbols(parent(a))
+   vars = Array{Int}(undef, length(kwargs))
+   vals = Array{RingElement}(undef, length(kwargs))
+   for (i, (var, val)) in enumerate(kwargs)
+     vari = findfirst(isequal(var), ss)
+     vari === nothing && error("Given polynomial has no variable $var")
+     vars[i] = vari
+     vals[i] = val
+   end
+   return evaluate(a, vars, vals)
+end
+
 ################################################################################
 #
 #  Derivative
@@ -1427,6 +1441,40 @@ function _map(g::T, p::MPolyRingElem, Rx) where {T}
    return finish(M)
 end
 
+###############################################################################
+#
+#  Universal polynomial ring methods
+#
+###############################################################################
+
+@doc raw"""
+    _upgrade(p::MPolyRingElem{T}, R::MPolyRing{T}) where {T}
+
+Return an element of `R` which is obtained from `p` by mapping the $i$-th variable
+of `parent(p)` to the $i$-th variable of `R`.
+For this to work, `R` needs to have at least as many variables as `parent(p)`.
+"""
+function _upgrade(p::MPolyRingElem{T}, R::MPolyRing{T}) where {T}
+   n = nvars(R) - nvars(parent(p))
+   n < 0 && error("Too few variables")
+   ctx = MPolyBuildCtx(R)
+   v0 = zeros(Int, n)
+   for (c, v) in zip(coefficients(p), exponent_vectors(p))
+      push_term!(ctx, c, vcat(v, v0))
+   end
+   return finish(ctx)
+end
+
+@doc raw"""
+    _add_gens(R::MPolyRing, varnames::Vector{Symbol})
+
+Return a new uncached multivariate polynomial ring which has the same properties
+as `R` but `varnames` as additional generators.
+"""
+function _add_gens(R::MPolyRing, varnames::Vector{Symbol})
+   return poly_ring(base_ring(R), vcat(symbols(R), varnames); internal_ordering=internal_ordering(R))
+end
+
 ###############################################################################
 #
 #  Factorization
@@ -1563,8 +1611,8 @@ julia> S, generators = polynomial_ring(ZZ, [:x, :y, :z])
 (Multivariate polynomial ring in 3 variables over integers, AbstractAlgebra.Generic.MPoly{BigInt}[x, y, z])
 ```
 """
-function polynomial_ring(R::Ring, s::Vector{Symbol}; kw...)
-   S = polynomial_ring_only(R, s; kw...)
+function polynomial_ring(R::Ring, s::Vector{Symbol}; cached::Bool=true, kw...)
+   S = poly_ring(R, s; cached, kw...)
    (S, gens(S))
 end
 
@@ -1650,10 +1698,14 @@ true
 :(@polynomial_ring)
 
 """
-    polynomial_ring_only(R::Ring, s::Vector{Symbol}; internal_ordering::Symbol=:lex, cached::Bool=true)
+    poly_ring(R::Ring, s::Vector{Symbol}; internal_ordering::Symbol=:lex, cached::Bool=false)
 
 Like [`polynomial_ring(R::Ring, s::Vector{Symbol})`](@ref) but return only the
-multivariate polynomial ring.
+multivariate polynomial ring. Moreover, the default value for `cached` is `false`,
+not `true`.
+
+This function is part of the internal interface for polynomial rings.
+User code should normally not need to invoke it.
 """
-polynomial_ring_only(R::T, s::Vector{Symbol}; internal_ordering::Symbol=:lex, cached::Bool=true) where T<:Ring =
+poly_ring(R::T, s::Vector{Symbol}; internal_ordering::Symbol=:lex, cached::Bool=false) where T<:Ring =
    mpoly_ring_type(T)(R, s, internal_ordering, cached)
