@@ -25,6 +25,10 @@ function is_exact_type(a::Type{T}) where {S <: RingElement, T <: PolyRingElem{S}
    return is_exact_type(S)
 end
 
+# ideal interface
+ideal_type(::Type{T}) where T <: PolyRing = Generic.Ideal{elem_type(T)}
+ideal(R::PolyRing{<:FieldElement}, V::Vector) = Generic.Ideal(R, V)
+
 @doc raw"""
     var(a::PolyRing)
 
@@ -48,7 +52,8 @@ Return the number of variables of the polynomial ring, which is 1.
 """
 number_of_variables(a::PolyRing) = 1
 
-characteristic(a::PolyRing) = characteristic(base_ring(a))
+characteristic(R::PolyRing) = characteristic(coefficient_ring(R))
+is_known(::typeof(characteristic), R::PolyRing) = is_known(characteristic, coefficient_ring(R))
 
 is_finite(a::PolyRing) = is_trivial(a)
 
@@ -1879,6 +1884,8 @@ function gcd(a::PolyRingElem{T}, b::PolyRingElem{T}, ignore_content::Bool = fals
       a = divexact(a, c1)
       b = divexact(b, c2)
       c = gcd(c1, c2)
+   else
+      c = nothing
    end
    lead_monomial = is_term_recursive(leading_coefficient(a)) ||
                    is_term_recursive(leading_coefficient(b))
@@ -1906,7 +1913,7 @@ function gcd(a::PolyRingElem{T}, b::PolyRingElem{T}, ignore_content::Bool = fals
          h = h^(1 - d)*g^d
       end
    end
-   if !ignore_content
+   if c !== nothing  # !ignore_content
       if !is_term_recursive(leading_coefficient(b)) &&
          !is_term_recursive(trailing_coefficient(b))
          if lead_monomial # lead term monomial, so content contains rest
@@ -3052,7 +3059,7 @@ end
 
 @doc raw"""
     power_sums_to_polynomial(P::Vector{T};
-                     parent::PolyRing{T}=PolyRing(parent(P[1])) where T <: RingElement -> PolyRingElem{T}
+                     parent::PolyRing{T}=poly_ring(parent(P[1])) where T <: RingElement -> PolyRingElem{T}
 
 Uses the Newton (or Newton-Girard) identities to obtain the polynomial
 with given sums of powers of roots. The list must be nonempty and contain
@@ -3060,7 +3067,7 @@ with given sums of powers of roots. The list must be nonempty and contain
 must start with the sum of first powers of the roots.
 """
 function power_sums_to_polynomial(P::Vector{T};
-                           parent::PolyRing{T}=PolyRing(parent(P[1]))) where T <: RingElement
+                           parent::PolyRing{T}=poly_ring(parent(P[1]))) where T <: RingElement
    return power_sums_to_polynomial(P, parent)
 end
 
@@ -3547,14 +3554,36 @@ end
 
 ###############################################################################
 #
-#   Polynomial Ring S, x = R[:x] syntax
+#   Syntactic sugar for Polynomial ring constructors
 #
 ###############################################################################
 
+# syntax: S, x = R[:x]
 getindex(R::NCRing, s::VarName) = polynomial_ring(R, s)
-# `R[:x, :y]` returns `S, [x, y]` instead of `S, x, y`
-getindex(R::NCRing, s::VarName, ss::VarName...) =
-   polynomial_ring(R, [Symbol(x) for x in (s, ss...)])
 
-# syntax: Rxy, y = R[:x][:y]
-getindex(R::Union{Tuple{PolyRing, PolyRingElem}, Tuple{NCPolyRing, NCPolyRingElem}}, s::VarName) = polynomial_ring(R[1], s)
+# syntax: S, (x,y) = R[:x,:y]
+getindex(R::Ring, s::VarName, t::VarName, ss::VarName...) = polynomial_ring(R, [Symbol(x) for x in (s, t, ss...)])
+
+# syntax: S, (x,y) = R[:x][:y]
+function getindex((R,x)::Union{Tuple{PolyRing, PolyRingElem}, Tuple{NCPolyRing, NCPolyRingElem}}, s::VarName)
+  S, y = polynomial_ring(R, s)
+  return S, elem_type(S)[S(x),y]
+end
+
+# syntax: S, (x,y,z) = R[:x][:y][:z] and S, (x,y,z) = R[:x,:y][:z]
+function getindex((R,xs)::Union{Tuple{PolyRing, Vector{<:PolyRingElem}}, Tuple{NCPolyRing, Vector{<:NCPolyRingElem}}, Tuple{MPolyRing, Vector{<:MPolyRingElem}}}, s::VarName)
+  S, y = polynomial_ring(R, s)
+  return S, elem_type(S)[S.(xs)...,y]
+end
+
+# syntax: S, (x,y,z) = R[:x][:y,:z]
+function getindex((R,x)::Tuple{PolyRing, PolyRingElem}, s::VarName, t::VarName, ss::VarName...)
+  S, ys = polynomial_ring(R, [Symbol(y) for y in (s, t, ss...)])
+  return S, elem_type(S)[S(x),ys...]
+end
+
+# syntax: S, (x,y,z,w) = R[:x][:y][:z,:w] and S, (x,y,z,w) = R[:x,:y][:z,:w]
+function getindex((R,xs)::Union{Tuple{PolyRing, Vector{<:PolyRingElem}}, Tuple{MPolyRing, Vector{<:MPolyRingElem}}}, s::VarName, t::VarName, ss::VarName...)
+  S, ys = polynomial_ring(R, [Symbol(y) for y in (s, t, ss...)])
+  return S, elem_type(S)[S.(xs)...,ys...]
+end
