@@ -35,7 +35,17 @@ behaviour of the function undefined if the division is not exact.
 """
 function divexact end
 
-divexact(x::RingElem, y::RingElem; check::Bool=true) = divexact(promote(x, y)...; check=check)
+function divexact(x::RingElem, y::RingElem; check::Bool=true)
+  xx, yy = promote(x, y)
+  # - if divexact is not implemented, we need to break the recursion
+  #   we assume that promotion returns identical operands if no proper
+  #   promotion can be performed
+  # - add type checks to make it constant-fold away in most cases
+  if typeof(xx) === typeof(x) && typeof(y) === typeof(yy) && (xx, yy) === (x, y)
+    throw(NotImplementedError(:divexact, x, y))
+  end
+  return divexact(xx, yy; check=check)
+end
 
 divexact(x::RingElem, y::RingElement; check::Bool=true) = divexact(x, parent(x)(y); check=check)
 
