@@ -9,10 +9,10 @@ function is_power(a::PolyRingElem, n::Int)
     # probably a equal-degree-factorisation would be good + some more gcd's
     # implement some Newton-type algo?
     degree(a) % n == 0 || return false, a
-    fl, x = is_power(leading_coefficient(a), n)
-    fl || return false, a
     f = factor(a)
     all(i % n == 0 for (_, i) in f) || return false, a
+    fl, x = is_power(constant_coefficient(f.unit), n)
+    fl || return false, a
     return true, x*prod(p^div(k, n) for (p, k) = f)
 end
 
@@ -23,7 +23,7 @@ end
 ################################################################################
 
 function factor(R::Field, f::PolyRingElem)
-    Rt = AbstractAlgebra.PolyRing(R)
+    Rt = AbstractAlgebra.poly_ring(R)
     f1 = change_base_ring(R, f; parent = Rt)
     return factor(f1)
 end
@@ -59,7 +59,8 @@ function roots(f::PolyRingElem)
     rts = Vector{elem_type(base_ring(f))}()
     for (p, e) in lf
         if degree(p) == 1
-            push!(rts, -divexact(constant_coefficient(p), leading_coefficient(p)))
+            fl, q = divides(constant_coefficient(p), leading_coefficient(p))
+            fl && push!(rts, -q)
         end
     end
     return rts
@@ -71,7 +72,7 @@ end
 Returns the roots of the polynomial `f` in the field `R` as an array.
 """
 function roots(R::Field, f::PolyRingElem)
-    Rt = AbstractAlgebra.PolyRing(R)
+    Rt = AbstractAlgebra.poly_ring(R)
     f1 = change_base_ring(R, f, parent = Rt)
     return roots(f1)
 end
@@ -90,10 +91,10 @@ function sturm_sequence(f::PolyRingElem{<:FieldElem})
         if r != 0
             push!(seq, -r)
             g, h = h, -r
-        else 
+        else
             break
         end
     end
     return seq
 end
- 
+
