@@ -48,6 +48,10 @@ function (a::EuclideanRingResidueRing{T})(b::Integer) where {T <: RingElement}
    return z
 end
 
+function (a::EuclideanRingResidueRing)(b::T) where {T <: Union{Rational, FracElem}}
+   return inv(a(denominator(b))) * a(numerator(b))
+end
+
 function (a::EuclideanRingResidueRing{T})(b::T) where {T <: RingElem}
    base_ring(a) !== parent(b) && error("Operation on incompatible objects")
    z = EuclideanRingResidueRingElem{T}(mod(b, modulus(a)))
@@ -110,8 +114,9 @@ function gens(R::Union{EuclideanRingResidueRing{T}, EuclideanRingResidueField{T}
 end
 
 function characteristic(R::Union{EuclideanRingResidueRing{T}, EuclideanRingResidueField{T}}) where {T<:PolyRingElem}
-   return characteristic(base_ring(base_ring(R)))
+   return characteristic(base_ring(R))
 end
+is_known(::typeof(characteristic), R::Union{EuclideanRingResidueRing{T}, EuclideanRingResidueField{T}}) where {T<:PolyRingElem} = is_known(characteristic, base_ring(R))
 
 function size(R::Union{EuclideanRingResidueRing{T}, EuclideanRingResidueField{T}}) where {T<:PolyRingElem}
    return size(base_ring(base_ring(R)))^degree(modulus(R))
