@@ -10,11 +10,11 @@
 #
 ###############################################################################
 
-base_ring_type(::Type{<:UniversalPolyRing{T}}) where T = parent_type(T)
-base_ring(S::UniversalPolyRing) = base_ring(mpoly_ring(S))::base_ring_type(S)
+base_ring_type(::Type{<:UniversalPolyRing{T}}) where T = mpoly_ring_type(T)
+base_ring(S::UniversalPolyRing) = S.base_ring::base_ring_type(S)
 
-coefficient_ring_type(T::Type{<:UniversalPolyRing}) = base_ring_type(T)
-coefficient_ring(S::UniversalPolyRing) = base_ring(S)
+coefficient_ring_type(::Type{<:UniversalPolyRing{T}}) where T = parent_type(T)
+coefficient_ring(S::UniversalPolyRing) = coefficient_ring(base_ring(S))::coefficient_ring_type(S)
 
 function is_domain_type(::Type{<:UnivPoly{S}}) where {S <: RingElement}
    return is_domain_type(S)
@@ -30,17 +30,13 @@ elem_type(::Type{UniversalPolyRing{T}}) where {T<:RingElement} = UnivPoly{T}
 
 parent_type(::Type{UnivPoly{T}}) where {T<:RingElement} = UniversalPolyRing{T}
 
-function mpoly_ring(S::UniversalPolyRing{T}) where {T<:RingElement}
-  return S.mpoly_ring::mpoly_ring_type(T)
-end
+number_of_variables(S::UniversalPolyRing) = number_of_variables(base_ring(S))
 
-number_of_variables(S::UniversalPolyRing) = number_of_variables(mpoly_ring(S))
+number_of_generators(S::UniversalPolyRing) = number_of_generators(base_ring(S))
 
-number_of_generators(S::UniversalPolyRing) = number_of_generators(mpoly_ring(S))
+symbols(S::UniversalPolyRing) = symbols(base_ring(S))
 
-symbols(S::UniversalPolyRing) = symbols(mpoly_ring(S))
-
-internal_ordering(p::UniversalPolyRing) = internal_ordering(mpoly_ring(p))
+internal_ordering(p::UniversalPolyRing) = internal_ordering(base_ring(p))
 
 data(p::UnivPoly{T}) where {T<:RingElement} = p.p::mpoly_type(T)
 
@@ -76,7 +72,7 @@ function coeff(p::UnivPoly, exps::Vector{Int})
    n = nvars(parent(data(p)))
    if len > n
       if !iszero(exps[n + 1:len])
-         return base_ring(S)()
+         return coefficient_ring(S)()
       end
       return coeff(data(p), exps[1:n])
    end
@@ -88,7 +84,7 @@ function coeff(p::UnivPoly, exps::Vector{Int})
 end
 
 function setcoeff!(p::UnivPoly, exps::Vector{Int}, c::T) where T <: RingElement
-   c = base_ring(data(p))(c)
+   c = coefficient_ring(data(p))(c)
    S = parent(p)
    len = length(exps)
    upgrade!(p)
@@ -110,9 +106,9 @@ end
 #
 ###############################################################################
 
-zero(R::UniversalPolyRing{T}) where {T} = UnivPoly{T}(zero(mpoly_ring(R)), R)
+zero(R::UniversalPolyRing{T}) where {T} = UnivPoly{T}(zero(base_ring(R)), R)
 
-one(R::UniversalPolyRing{T}) where {T} = UnivPoly{T}(one(mpoly_ring(R)), R)
+one(R::UniversalPolyRing{T}) where {T} = UnivPoly{T}(one(base_ring(R)), R)
 
 iszero(p::UnivPoly) = iszero(data(p))
 
@@ -230,8 +226,7 @@ function _ensure_variables(S::UniversalPolyRing, v::Vector{<:VarName})
       end
    end
    if !isempty(added_symbols)
-      new_symbols = vcat(current_symbols, added_symbols)
-      S.mpoly_ring = AbstractAlgebra.polynomial_ring_only(base_ring(S), new_symbols; internal_ordering=internal_ordering(S), cached=false)
+      S.base_ring = AbstractAlgebra._add_gens(base_ring(S), added_symbols)
    end
    return idx
 end
@@ -239,17 +234,15 @@ end
 function gen(S::UniversalPolyRing, s::VarName)
    i = findfirst(==(Symbol(s)), symbols(S))
    if i === nothing
-      new_symbols = copy(symbols(S))
-      push!(new_symbols, Symbol(s))
-      i = length(new_symbols)
-      S.mpoly_ring = AbstractAlgebra.polynomial_ring_only(base_ring(S), new_symbols; internal_ordering=internal_ordering(S), cached=false)
+      S.base_ring = AbstractAlgebra._add_gens(base_ring(S), [Symbol(s)])
+      i = length(symbols(S))
    end
    return @inbounds gen(S, i)
 end
 
 function gen(S::UniversalPolyRing{T}, i::Int) where {T}
    @boundscheck 1 <= i <= nvars(S) || throw(ArgumentError("generator index out of range"))
-   return UnivPoly{T}(gen(mpoly_ring(S), i), S)
+   return UnivPoly{T}(gen(base_ring(S), i), S)
 end
 
 function gens(S::UniversalPolyRing{T}) where {T}
@@ -263,7 +256,7 @@ function _univ_poly_gens(S::UniversalPolyRing{T}, vars::Vector{Symbol}) where {T
    idx = _ensure_variables(S, vars)
    # TRICK: @varnames_interface expects two return values, but we only care
    # for the second; so just return literally nothing for the first
-   return nothing, [UnivPoly{T}(gen(mpoly_ring(S), i), S) for i in idx]
+   return nothing, [UnivPoly{T}(gen(base_ring(S), i), S) for i in idx]
 end
 
 AbstractAlgebra.@varnames_interface _univ_poly_gens(R::UniversalPolyRing{T}, s) where {T}
@@ -286,7 +279,8 @@ end
 
 canonical_unit(p::UnivPoly) = canonical_unit(data(p))
 
-characteristic(R::UniversalPolyRing) = characteristic(base_ring(R))
+characteristic(R::UniversalPolyRing) = characteristic(coefficient_ring(R))
+is_known(::typeof(characteristic), R::UniversalPolyRing) = is_known(characteristic, coefficient_ring(R))
 
 function Base.hash(p::UnivPoly, h::UInt)
    b = 0xcf418d4529109236%UInt
@@ -349,7 +343,7 @@ function show(io::IO, R::UniversalPolyRing)
    @show_name(io, R)
    @show_special(io, R)
    print(io, "Universal Polynomial Ring over ")
-   show(io, base_ring(R))
+   show(io, coefficient_ring(R))
 end
 
 function expressify(a::UnivPoly, x = symbols(parent(a)); context = nothing)
@@ -363,7 +357,7 @@ end
 #   Unary operations
 #
 ###############################################################################
-   
+
 function -(p::UnivPoly{T}) where {T}
    return UnivPoly{T}(-data(p), parent(p))
 end
@@ -752,93 +746,26 @@ end
 #
 ###############################################################################
 
-function evaluate(a::UnivPoly{T}, A::Vector{T}) where {T <: RingElem}
-   R = base_ring(a)
+function evaluate(a::UnivPoly, A::Vector{<:NCRingElement})
+   isempty(A) && error("Evaluating at an empty list of values is not allowed")
+   a2 = data(a)
+   varidx = var_indices(a2)
+   isempty(varidx) && return constant_coefficient(a2) # TODO: this is weird
+   vals = [zero(parent(A[1])) for _ in 1:nvars(parent(a2))]
    n = length(A)
-   num = nvars(parent(data(a)))
-   if n > num
-      n > nvars(parent(a)) && error("Too many values")
-      if nvars(parent(data(a))) == 0
-         return constant_coefficient(data(a))*one(parent(A[1]))
-      end
-      return evaluate(data(a), A[1:num])
+   for i in varidx
+      i <= n || error("Number of variables does not match number of values")
+      vals[i] = A[i]
    end
-   if n < num
-      A = vcat(A, [zero(R) for i = 1:num - n])
-   end
-   return evaluate(data(a), A)
+   return evaluate(a2, vals)
 end
 
-function evaluate(a::UnivPoly{T}, A::Vector{V}) where {T <: RingElement, V <: JuliaRingElement}
-   n = length(A)
-   num = nvars(parent(data(a)))
-   if n > num
-      n > nvars(parent(a)) && error("Too many values")
-      if nvars(parent(data(a))) == 0
-         return constant_coefficient(data(a))*one(parent(A[1]))
-      end
-      return evaluate(data(a), A[1:num])
-   end
-   if n < num
-      A = vcat(A, zeros(V, num - n))
-   end
-   return evaluate(data(a), A)
-end
-
-function evaluate(a::UnivPoly{T}, A::Vector{V}) where {T <: RingElement, V <: RingElement}
-   n = length(A)
-   num = nvars(parent(data(a)))
-   if n > num
-      n > nvars(parent(a)) && error("Too many values")
-      if nvars(parent(data(a))) == 0
-         return constant_coefficient(data(a))*one(parent(A[1]))
-      end
-      return evaluate(data(a), A[1:num])
-   end
-   if n < num
-      if n == 0
-         R = base_ring(a)
-         return evaluate(data(a), [zero(R) for _ in 1:num])
-      else
-         R = parent(A[1])
-         A = vcat(A, [zero(R) for _ in 1:num-n])
-         return evaluate(data(a), A)
-     end
-   end
-   return evaluate(data(a), A)
-end
-
-function (a::UnivPoly{T})() where {T <: RingElement}
-   return evaluate(a, T[])
-end
-
-function (a::UnivPoly{T})(vals::T...) where {T <: RingElement}
-   return evaluate(a, [vals...])
-end
-
-function (a::UnivPoly{T})(val::V, vals::V...) where {T <: RingElement, V <: JuliaRingElement}
+function (a::UnivPoly)(val::T, vals::T...) where T <: NCRingElement
    return evaluate(a, [val, vals...])
 end
 
-function (a::UnivPoly{T})(vals::NCRingElement...) where {T <: RingElement}
-   A = [vals...]
-   n = length(vals)
-   num = nvars(parent(data(a)))
-   if n > num
-      n > nvars(parent(a)) && error("Too many values")
-      if nvars(parent(data(a))) == 0
-         return constant_coefficient(data(a))*one(parent(A[1]))
-      end
-      return data(a)(vals[1:num]...)
-   end
-   if n < num
-      A = vcat(A, zeros(Int, num - n))
-   end
-   return data(a)(A...)
-end
-
-function evaluate(a::UnivPoly{T}, vals::Vector{V}) where {T <: RingElement, V <: NCRingElem}
-   return a(vals...)
+function (a::UnivPoly)()
+   return evaluate(a, Int[])
 end
 
 function evaluate(a::UnivPoly{T}, vars::Vector{Int}, vals::Vector{V}) where {T <: RingElement, V <: RingElement}
@@ -847,15 +774,15 @@ function evaluate(a::UnivPoly{T}, vars::Vector{Int}, vals::Vector{V}) where {T <
    vals2 = Vector{mpoly_type(T)}(undef, 0)
    num = nvars(parent(data(a)))
    S = parent(a)
-   n = nvars(S)
+   upgrade!(a)
+   a2 = data(a)
    for i = 1:length(vars)
-      vars[i] > n && error("Unknown variable")
       if vars[i] <= num
          push!(vars2, vars[i])
          push!(vals2, data(S(vals[i])))
       end
    end
-   return UnivPoly(evaluate(data(S(a)), vars2, vals2), S)
+   return UnivPoly(evaluate(a2, vars2, vals2), S)
 end
 
 function evaluate(a::S, vars::Vector{S}, vals::Vector{V}) where {S <: UnivPoly{T}, V <: RingElement} where {T <: RingElement}
@@ -863,19 +790,6 @@ function evaluate(a::S, vars::Vector{S}, vals::Vector{V}) where {S <: UnivPoly{T
    return evaluate(a, varidx, vals)
 end
 
-function (a::Union{MPolyRingElem, UniversalPolyRingElem})(;kwargs...)
-   ss = symbols(parent(a))
-   vars = Array{Int}(undef, length(kwargs))
-   vals = Array{RingElement}(undef, length(kwargs))
-   for (i, (var, val)) in enumerate(kwargs)
-     vari = findfirst(isequal(var), ss)
-     vari === nothing && error("Given polynomial has no variable $var")
-     vars[i] = vari
-     vals[i] = val
-   end
-   return evaluate(a, vars, vals)
-end
-
 ########S,(a,b)=QQ[:a,:b]#######################################################################
 #
 #   GCD
@@ -908,7 +822,7 @@ is_univariate(p::UnivPoly) = is_univariate(data(p))
 
 is_univariate_with_data(p::UnivPoly) = is_univariate_with_data(data(p))
 
-is_univariate(R::UniversalPolyRing) = is_univariate(mpoly_ring(R))
+is_univariate(R::UniversalPolyRing) = is_univariate(base_ring(R))
 
 function coefficients_of_univariate(p::UnivPoly, check_univariate::Bool=true)
    return coefficients_of_univariate(data(p), check_univariate)
@@ -924,7 +838,7 @@ _change_univ_poly_ring(R, Rx, cached::Bool) = universal_polynomial_ring(R, symbo
 
 function change_base_ring(R::Ring, p::UnivPoly{T}; cached::Bool=true, parent::UniversalPolyRing = _change_univ_poly_ring(R, parent(p), cached)) where {T <: RingElement}
    upgrade!(p)
-   return UnivPoly(change_base_ring(R, data(p); parent = mpoly_ring(parent)), parent)
+   return UnivPoly(change_base_ring(R, data(p); parent = base_ring(parent)), parent)
 end
 
 function change_coefficient_ring(R::Ring, p::UnivPoly{T}; cached::Bool=true, parent::UniversalPolyRing = _change_univ_poly_ring(R, parent(p), cached)) where {T <: RingElement}
@@ -937,9 +851,9 @@ end
 #
 ################################################################################
 
-function map_coefficients(f::T, p::UnivPoly; cached::Bool=true, parent::UniversalPolyRing = _change_univ_poly_ring(parent(f(zero(base_ring(p)))), parent(p), cached)) where T
+function map_coefficients(f::T, p::UnivPoly; cached::Bool=true, parent::UniversalPolyRing = _change_univ_poly_ring(parent(f(zero(coefficient_ring(p)))), parent(p), cached)) where T
    upgrade!(p)
-   return UnivPoly(map_coefficients(f, data(p); parent = mpoly_ring(parent)), parent)
+   return UnivPoly(map_coefficients(f, data(p); parent = base_ring(parent)), parent)
 end
 
 ###############################################################################
@@ -964,42 +878,24 @@ end
 #
 ###############################################################################
 
-RandomExtensions.maketype(S::AbstractAlgebra.UniversalPolyRing, _, _, _) = elem_type(S)
+RandomExtensions.maketype(S::AbstractAlgebra.UniversalPolyRing, _...) = elem_type(S)
 
-function RandomExtensions.make(S::AbstractAlgebra.UniversalPolyRing, term_range::AbstractUnitRange{Int},
-                               exp_bound::AbstractUnitRange{Int}, vs...)
-   R = base_ring(S)
-   if length(vs) == 1 && elem_type(R) == Random.gentype(vs[1])
-      Make(S, term_range, exp_bound, vs[1])
-   else
-      Make(S, term_range, exp_bound, make(R, vs...))
-   end
+function RandomExtensions.make(S::AbstractAlgebra.UniversalPolyRing, vs...)
+   return Make(S, make(base_ring(S), vs...))
 end
 
-function rand(rng::AbstractRNG, sp::SamplerTrivial{<:Make4{
-                 <:RingElement, <:AbstractAlgebra.UniversalPolyRing, <:AbstractUnitRange{Int}, <:AbstractUnitRange{Int}}})
-   S, term_range, exp_bound, v = sp[][1:end]
-   f = S()
-   g = gens(S)
-   R = base_ring(S)
-   for i = 1:rand(rng, term_range)
-      term = S(1)
-      for j = 1:length(g)
-         term *= g[j]^rand(rng, exp_bound)
-      end
-      term *= rand(rng, v)
-      f += term
-   end
-   return f
+function rand(rng::AbstractRNG, sp::SamplerTrivial{<:Make2{
+                 <:RingElement, <:AbstractAlgebra.UniversalPolyRing}})
+   S, v = sp[][1:end]
+   return UnivPoly(rand(rng, v), S)
 end
 
-function rand(rng::AbstractRNG, S::AbstractAlgebra.UniversalPolyRing,
-              term_range::AbstractUnitRange{Int}, exp_bound::AbstractUnitRange{Int}, v...)
-   rand(rng, make(S, term_range, exp_bound, v...))
+function rand(rng::AbstractRNG, S::AbstractAlgebra.UniversalPolyRing, v...)
+   return rand(rng, make(S, v...))
 end
 
-function rand(S::AbstractAlgebra.UniversalPolyRing, term_range, exp_bound, v...)
-   rand(Random.default_rng(), S, term_range, exp_bound, v...)
+function rand(S::AbstractAlgebra.UniversalPolyRing, v...)
+   return rand(Random.default_rng(), S, v...)
 end
 
 ###############################################################################
@@ -1126,39 +1022,28 @@ end
 #
 ###############################################################################
 
-function upgrade(S::UniversalPolyRing{T}, pp::MPolyRingElem{T}) where {T}
-   n = nvars(S) - nvars(parent(pp))
-   if n > 0
-      ctx = MPolyBuildCtx(mpoly_ring(S))
-      v0 = zeros(Int, n)
-      for (c, v) in zip(coefficients(pp), exponent_vectors(pp))
-         push_term!(ctx, c, vcat(v, v0))
-      end
-      return finish(ctx)
-   else
-      return pp
-   end
-end
-
 function upgrade!(p::UnivPoly)
-   p.p = upgrade(parent(p), data(p))
+   R = base_ring(parent(p))
+   if R != parent(p.p)
+      p.p = AbstractAlgebra._upgrade(p.p, R)
+   end
    return p
 end
 
 function (a::UniversalPolyRing{T})(b::RingElement) where {T <: RingElement}
-   return a(base_ring(a)(b))
+   return a(coefficient_ring(a)(b))
 end
 
 function (a::UniversalPolyRing{T})() where {T <: RingElement}
-   return UnivPoly{T}(mpoly_ring(a)(), a)
+   return UnivPoly{T}(base_ring(a)(), a)
 end
 
 function (a::UniversalPolyRing{T})(b::JuliaRingElement) where {T <: RingElement}
-   return UnivPoly{T}(mpoly_ring(a)(b), a)
+   return UnivPoly{T}(base_ring(a)(b), a)
 end
 
 function (a::UniversalPolyRing{T})(b::T) where {T <: RingElem}
-   return UnivPoly{T}(mpoly_ring(a)(b), a)
+   return UnivPoly{T}(base_ring(a)(b), a)
 end
 
 function (S::UniversalPolyRing{T})(p::UnivPoly{T}) where {T <: RingElement}
@@ -1169,12 +1054,12 @@ end
 function (a::UniversalPolyRing{T})(b::Vector{T}, m::Vector{Vector{Int}}) where {T <: RingElement}
    if length(m) != 0
       len = length(m[1])
-      num = nvars(mpoly_ring(a))
+      num = nvars(base_ring(a))
       if len != num
          for i = 1:length(m)
             m[i] = vcat(m[i], zeros(Int, num - len))
          end
       end
    end
-   return UnivPoly{T}(mpoly_ring(a)(b, m), a)
+   return UnivPoly{T}(base_ring(a)(b, m), a)
 end
