@@ -105,13 +105,6 @@ end
 
    @test isa(l, Generic.LaurentSeriesElem)
 
-   @test x in [x, y]
-   @test x in [y, x]
-   @test !(x in [y])
-
-   @test x in keys(Dict(x => 1))
-   @test !(y in keys(Dict(x => 1)))
-   
    R, x = laurent_series_ring(ZZ, 30, "x")
    RR, x = laurent_series_ring(ZZ, 30, "x")
    @test R === RR
@@ -311,7 +304,7 @@ end
    @test isa(r, Generic.LaurentSeriesFieldElem)
 
    s = laurent_series(ZZ, [1, 2, 3], 3, 5, 0, 1; max_precision=10)
-   
+
    @test max_precision(parent(s)) == 10
 end
 
