@@ -23,6 +23,10 @@
    @test elem_type(S3) == Generic.NCPoly{Generic.NCPoly{Generic.MatRingElem{BigInt}}}
    @test typeof(S3) == Generic.NCPolyRing{Generic.NCPoly{Generic.MatRingElem{BigInt}}}
 
+   S4, _ = R["x"]["y"]["z"]
+   @test elem_type(S4) == Generic.NCPoly{Generic.NCPoly{Generic.NCPoly{Generic.MatRingElem{BigInt}}}}
+   @test typeof(S4) == Generic.NCPolyRing{Generic.NCPoly{Generic.NCPoly{Generic.MatRingElem{BigInt}}}}
+
    S, y = S1
    T, z = polynomial_ring(S, "z")
 
@@ -437,7 +441,7 @@ end
 
 @testset "Generic.NCPoly.map_coefficients" begin
    let
-      Q = matrix_ring(QQ, 2) 
+      Q = matrix_ring(QQ, 2)
       Qz, z = Q[:z]
       Qx, x = QQ[:x]
       f = map_coefficients(c -> c[1, 1], 2*z; parent = Qx)
