@@ -58,16 +58,6 @@
 
    @test isa(g, EuclideanRingResidueFieldElem)
 
-   S, = Generic.residue_ring(B, 2)
-   x = R(1)
-   y = S(1)
-   @test x in [x, y]
-   @test x in [y, x]
-   @test !(x in [y])
-
-   @test x in keys(Dict(x => 1))
-   @test !(y in keys(Dict(x => 1)))
-
    # trivial rings can not be fields
    @test_throws ArgumentError residue_field(ZZ, 1)
    R = residue_ring(ZZ, 1)[1]
